#!/bin/sh
#
# Name: wfc3_undo.s
#
# Purpose: will "undo" a CDBS reference file by making it appear the
#          file is rejected (=inactive).  A specific WFC3 version is needed
#          in order to list out the wfc3_row table columns explicitly,
#          due to the use of an IDENTITY column in that table to create
#          a unique key with less than 16 fields.
#
# Using delivery number as a proxy from version number of file.
#
# Called by: nothing (stand-alone tool)
#
# Usage:
#   (MAKE SURE that the CDBS_SERVER , CDBS_DB
#                       REFFILE_SERVER, and REFFILE_DB environment variables
#    are pointing at the database you want to alter)
#
#   wfc3_undo.s wfc3 REF_FILE_NAME
#        (yes, the 1st "wfc3" argument is redundant, but since that allows
#         the tool to follow the same form as the non-wfc3 version, it remains)
#
# History:
# 03/01/2011  S.Lubow    first version
# 04/07/2011  MSwam      added command-line arg verification
# 05/18/2011  MSwam      reworked to deal with IDENTITY column in wfc3_row
#---------------------------------------------------------------------------
if [ -z "$1" ]
then
   echo "Usage ERROR: first argument should be instrument prefix (e.g. acs)."
   exit 1
fi
# lowercase the instrument for db table names
instrument=`echo $1 | tr '[A-Z]' '[a-z]'`
#
if [ -z "$2" ]
then
   echo "Usage ERROR: 2nd argument should be reference file name (e.g. K9C13372J_BPX.FITS)."
   exit 1
fi
# uppercase the reffile name for DB comparisons
reffile_name=`echo $2 | tr '[a-z]' '[A-Z]'`
#
echo "first attempting the UNDO operations in the CDBS_SERVER,CDBS_DB database..."
echo " "

isql -S$CDBS_SERVER -Umswamdbo << EOF
use $CDBS_DB
go
set IDENTITY_INSERT wfc3_row ON
go

create table #exp (n int)
go
select top 0 * into #tf from ${instrument}_file
select top 0 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
   into #tr from ${instrument}_row
go

begin transaction
if not exists (select * from cdbs_mode where lower(instrument) = '${instrument}')
begin
 print '-> CDBS ERROR:'
 print '->  NO SUCH INSTRUMENT'
 print '-> MUST USE:'
 select distinct '->',  lower(instrument) from cdbs_mode
 goto done
end

if not exists (select * from ${instrument}_file where file_name = '${reffile_name}')
begin
 print '-> CDBS ERROR:'
 print '->  NO SUCH FILE'
 goto done
end

if not exists (select * from ${instrument}_file where file_name = '${reffile_name}' 
      and reject_flag='N')
begin
 print '-> CDBS ERROR:'
 print '->  FILE ALREADY REJECTED'
 goto done
end

-- store expansion numbers to be rejected
insert into #exp(n)
select expansion_number from ${instrument}_file
where file_name='${reffile_name}' and reject_flag='N'

-- create new delivery with reactivated files
update delivery_number set value=value+1 
    where value = (select max(value) from delivery_number)

declare @n int
declare @d datetime
select @d=getdate()
declare @dm int
declare @f varchar(256)

-- loop through rejection by expansion number
while exists (select * from #exp)
begin
  select @n = min(n) from #exp
  update ${instrument}_file set reject_flag='Y', reject_by_file_name='${reffile_name}',
   reject_by_expansion_number = @n,
   reject_delivery_number=(select max(value) from delivery_number),
   comment = 'Rejected by undo software ' + convert(varchar(200), @d) + '. ' 
             + convert(varchar(8000),comment) 
    where file_name='${reffile_name}' and reject_flag='N' and expansion_number=@n
  select '->  Rejecting file ${reffile_name}', @n
  delete from #exp where n=@n
end
delete from #exp

-- loop through activation of files 
if exists (select * from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}')
begin
  select @dm = max(delivery_number) from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}'
  select @f = file_name from ${instrument}_file where file_name in
      (select file_name from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}') and
      delivery_number = @dm
  insert into #exp(n)
  select expansion_number from ${instrument}_file 
      where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}' and delivery_number=@dm 

  while exists (select * from #exp)
  begin
   select @n = min(n) from #exp
   insert  into #tr select 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
from ${instrument}_row where file_name = @f and
      delivery_number = @dm and
	  expansion_number = @n
   update #tr set delivery_number = (select max(value) from delivery_number)
   insert ${instrument}_row select 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
   from #tr

   insert into #tf select * from ${instrument}_file where file_name = @f and
      delivery_number = @dm and
	  expansion_number = @n
   update #tf set delivery_number = (select max(value) from delivery_number),
      reject_flag='N', reject_by_file_name=null, 
      reject_delivery_number=null,
      comment = 'Reactivated by undo software ' + convert(varchar(200),@d) + '. ' 
             + convert(varchar(8000),comment)
   insert ${instrument}_file select * from #tf
   
   print '->  Activating rejected file'
   select '-> ', file_name, @n from ${instrument}_file where reject_by_file_name = '${reffile_name}'  and file_name <> '${reffile_name}' 
              and expansion_number=@n and delivery_number = @dm
   delete from #exp where n=@n
   delete from #tr
   delete from #tf
  end
end

--select * from ${instrument}_file where 
-- reject_delivery_number=(select max(value) from delivery_number)

--select * from ${instrument}_file where delivery_number=(select max(value) from delivery_number) 

done:
-- rollback transaction
commit transaction
go
quit
EOF

######################
echo "and now the same operations for the REFFILE_SERVER,REFFILE_DB..."
# the DB responses should be identical, as the *_file and *_row tables
# in these 2 databases should be kept in sync
######################

isql -S$REFFILE_SERVER -Umswamdbo << EOF2
use $REFFILE_DB
go
	
create table #exp (n int)
go
select top 0 * into #tf from ${instrument}_file
select top 0 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
into #tr from ${instrument}_row
go

begin transaction

if not exists (select * from ${instrument}_file where file_name = '${reffile_name}')
begin
 print '-> CDBS ERROR:'
 print '->  NO SUCH FILE'
 goto done
end

if not exists (select * from ${instrument}_file where file_name = '${reffile_name}' 
      and reject_flag='N')
begin
 print '-> CDBS ERROR:'
 print '->  FILE ALREADY REJECTED'
 goto done
end

-- store expansion numbers to be rejected
insert into #exp(n)
select expansion_number from ${instrument}_file
where file_name='${reffile_name}' and reject_flag='N'

-- create new delivery with reactivated files
update delivery_number set value=value+1 
    where value = (select max(value) from delivery_number)

declare @n int
declare @d datetime
select @d=getdate()
declare @dm int
declare @f varchar(256)

-- loop through rejection by expansion number
while exists (select * from #exp)
begin
  select @n = min(n) from #exp
  update ${instrument}_file set reject_flag='Y', reject_by_file_name='${reffile_name}',
   reject_by_expansion_number = @n,
   reject_delivery_number=(select max(value) from delivery_number),
   comment = 'Rejected by undo software ' + convert(varchar(200), @d) + '. ' 
             + convert(varchar(8000),comment) 
    where file_name='${reffile_name}' and reject_flag='N' and expansion_number=@n
  select '->  Rejecting file ${reffile_name}', @n
  delete from #exp where n=@n
end
delete from #exp

-- loop through activation of files 
if exists (select * from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}')
begin
  select @dm = max(delivery_number) from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}'
  select @f = file_name from ${instrument}_file where file_name in
      (select file_name from ${instrument}_file where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}') and
      delivery_number = @dm
  insert into #exp(n)
  select expansion_number from ${instrument}_file 
      where reject_by_file_name = '${reffile_name}' and file_name <> '${reffile_name}' and delivery_number=@dm 

  while exists (select * from #exp)
  begin
   select @n = min(n) from #exp
   insert  into #tr select 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
   from ${instrument}_row where file_name = @f and
      delivery_number = @dm and
	  expansion_number = @n
   update #tr set delivery_number = (select max(value) from delivery_number)
   insert ${instrument}_row select 
   file_name,
   expansion_number,
   delivery_number,
   detector,
   ccdamp,
   ccdgain,
   binaxis1,
   binaxis2,
   samp_seq,
   subtype,
   filter,
   ccdchip,
   ccdofsta,
   ccdofstb,
   ccdofstc,
   ccdofstd,
   crsplit,
   binx,
   biny,
   detchip,
   meanexp,
   aperture,
   direction,
   equivalence_class_severe,
   equivalence_class_moderate,
   pedigree,
   observation_begin_date,
   observation_end_date,
   comment 
   from #tr

   insert into #tf select * from ${instrument}_file where file_name = @f and
      delivery_number = @dm and
	  expansion_number = @n
   update #tf set delivery_number = (select max(value) from delivery_number),
      reject_flag='N', reject_by_file_name=null, 
      reject_delivery_number=null,
      comment = 'Reactivated by undo software ' + convert(varchar(200),@d) + '. ' 
             + convert(varchar(8000),comment)
   insert ${instrument}_file select * from #tf
   
   print '->  Activating rejected file'
   select '-> ', file_name, @n from ${instrument}_file where reject_by_file_name = '${reffile_name}'  and file_name <> '${reffile_name}' 
              and expansion_number=@n and delivery_number = @dm
   delete from #exp where n=@n
   delete from #tr
   delete from #tf
  end
end

--select * from ${instrument}_file where 
-- reject_delivery_number=(select max(value) from delivery_number)

--select * from ${instrument}_file where delivery_number=(select max(value) from delivery_number) 

done:
-- rollback transaction
commit transaction
go
set IDENTITY_INSERT wfc3_row OFF
go
quit
EOF2
exit
